
%option noyywrap
%option yylineno
%option nounput
%option noinput

WHITESPACE      [[:space:]]
INTEGER         [[:digit:]]+
FLOAT           ({INTEGER}"."{INTEGER})|("."{INTEGER})
VARIABLE        [[:alpha:]][[:alnum:]_]*

%x COMMENT

%{
#include "minako.h"
#include <string.h>
#include <stdarg.h>
	
	extern void yyerror(const char* message, ...);
%}

%%

{WHITESPACE}+    { /* ignoriere Leerzeichen */ }
"//".*\n         { /* C++ Kommentar */ }
"/*"             { BEGIN(COMMENT); }
<COMMENT>"*/"    { BEGIN(INITIAL); }
<COMMENT>.|\n    { /* ignoriere den Text des C Kommentars */ }
<COMMENT><<EOF>> { yyerror("offener Kommentar am Ende des Quellcodes"); }

"&&"             { return AND; }
"||"             { return OR; }
"=="             { return EQ; }
"!="             { return NEQ; }
"<="             { return LEQ; }
">="             { return GEQ; }
"\<"             { return LSS; }
">"              { return GRT; }

"bool"           { return KW_BOOLEAN; }
"do"             { return KW_DO; }
"else"           { return KW_ELSE; }
"float"          { return KW_FLOAT; }
"for"            { return KW_FOR; }
"if"             { return KW_IF; }
"int"            { return KW_INT; }
"printf"         { return KW_PRINTF; }
"return"         { return KW_RETURN; }
"void"           { return KW_VOID; }
"while"          { return KW_WHILE; }

{FLOAT}([eE][\-+]?{INTEGER})? |
{INTEGER}[eE][\-+]?{INTEGER}  { yylval.floatValue = atof(yytext);
                                return CONST_FLOAT; }
{INTEGER}                     { yylval.intValue = atoi(yytext);
                                return CONST_INT; }
"true"                        { yylval.intValue = 1;
                                return CONST_BOOLEAN; }
"false"                       { yylval.intValue = 0;
                                return CONST_BOOLEAN; }
"\""[^[:cntrl:]\"]*"\""       { yylval.string = strdup(yytext);
                                return CONST_STRING; }

{VARIABLE}                    { yylval.string = strdup(yytext);
                                return ID; }
[+\-/=*,;(){}]                { return *yytext; }
<<EOF>>                       { return EOF; }
.                             { yyerror("Lexikalischer Fehler bei '%c'",
                                        *yytext); }

%%

void yyerror(const char* message, ...)
{
	va_list args;
	
	va_start(args, message);
	fprintf(stderr, "Fehler in Zeile %2d: ", yylineno);
	vfprintf(stderr, message, args);
	fprintf(stderr, "\n");
	va_end(args);
	
	exit(-1);
}
